# -*- coding: utf-8 -*-
"""
Created on Mon April 2 11:10:20 2023

@author: Arnaud DUGAS, Lycée Livet, Nantes, arnaud.dugas@ac-nantes.fr

This file is part of HacheurXX.

Foobar is free software: you can redistribute it and/or modify it under the terms of
the GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

HacheurXX is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Foobar.
If not, see <https://www.gnu.org/licenses/>. s

"""

# Importation :
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Slider, Button, RadioButtons
from scipy.integrate import odeint  # pour la résolution de l'équadiff

version = 1.9  # du 13/04/2023
mode = "unipolaire" #unipolaire ou bipolaire
mode_fluide = False # Permet de fluidifier l'affichage si True (On actualise plus les chronogrames pendant le déplacement du curseur)

if mode == "unipolaire":
    titre = "Hacheur en commande unipolaire"
else:
    titre = "Hacheur en commande bipolaire"

# Paramètres du hacheur
T = 100  # période en ms
alpha = 0.5  # rapport cyclique (recalculé en fonction du pt de fonctionnement)

# Paramètres du moteur
R = 1  # résistance ohm
L = 10  # inductance mH

# Limites des diagrammes
Umin, Umax = -5, 5  # en A
Imin, Imax = -5, 5  # en V
Tmax = 1000  # Tmax en ms
N = 501  # Nb points diagrammes U=f(t) et I=f(t)
         # 501 par défaut, on peut augmenter cette valeur au détriment de la fluidité

# Point de fonctionnement
Uc, Ic = 1.9, 1.8  # Point de fonctionnement

# Curseur des diagrammes U=f(t) et I=f(t)
tc = Tmax/2  # temps du curseur correspondant
nc = int((tc/Tmax)*N)  # Indice du curseur du le diagramme U et I

# Stockage des données
Tt = np.zeros(N)  # Tableau des temps
Tu = np.zeros(N)  # Tableau des tensions
Ti = np.zeros(N)  # Tableau des intensités

# Données temproraires pour le calcul
# On calcul 2*N pts pour garder les N derniers pour s'affranchir du régime transitoire
Tt_c = np.zeros(2*N)  # Tableau des temps 
Tu_c = np.zeros(2*N)  # Tableau des tensions
Ti_c = np.zeros(2*N)  # Tableau des intensités

# Diagramme UI (Point de fonctionnement)
class UIdiagram:
    def __init__(self, ax):
        self.ax = ax
        # plt.axis('equal')
        ax.set_xlim(-5, 5)
        ax.set_ylim(-5, 5)
        plt.xlabel("<uc> en V")
        plt.ylabel("<ic> en A")
        self.point, = ax.plot(Uc, Ic, 'bo')
        ax.plot([Umin, Umax], [0, 0], 'k')
        ax.plot([0, 0], [Imin, Imax], 'k')
        self.txt = ax.text(-5, 3, '',)
        self.txt.set_text('<uc>=%1.2f V, <ic>=%1.2f A' % (Uc, Ic))
        self.info = ax.text(0, -4, '', horizontalalignment='center',
                            bbox=dict(facecolor='red', alpha=0.5))
        self.info.set_text(
            'Déplacer à la souris le point\n de fonctionnement\ndans les quadrants')
        self.press = None

    def connect(self):
        """connect to all the events we need"""
        self.cidpress = self.point.figure.canvas.mpl_connect(
            'button_press_event', self.on_press)
        self.cidrelease = self.point.figure.canvas.mpl_connect(
            'button_release_event', self.on_release)
        self.cidmotion = self.point.figure.canvas.mpl_connect(
            'motion_notify_event', self.on_motion)

    def on_press(self, event):
        """on button press we will see if the mouse is over us and store some data"""
        #print("on press")
        if event.inaxes != self.point.axes:
            return

        contains, attrd = self.point.contains(event)
        if not contains:
            return
        reset_dr1()
        self.press = event.xdata, event.ydata
        self.info.set_text('')

    def on_motion(self, event):
        """on motion we will move the point if the mouse is over us"""
        global Uc, Ic

        if self.press is None:
            return
        if event.inaxes != self.point.axes:
            return
        #xpress, ypress = self.press
        Uc = event.xdata
        Ic = event.ydata
        #print('{},{}'.format(event.xdata, event.ydata))
        self.point.set_xdata(Uc)
        self.point.set_ydata(Ic)
        self.txt.set_text('<uc>=%1.2f V, <ic>=%1.2f A' % (Uc, Ic))
        if not(mode_fluide):
            updateAll() # Désactivé pour limiter les calculs
        update_dr5()  # Mise à jour du schéma méca
        self.point.figure.canvas.draw()

    def on_release(self, event):
        """on release we reset the press data"""
        self.press = None
        self.info.set_text(
            'Déplacer à la souris le point\n de fonctionnement\ndans les quadrants')
        self.point.figure.canvas.draw()
        if mode_fluide:
            updateAll()
        update_dr1()  # Mise à jour du schéma élec

    def disconnect(self):
        """disconnect all the stored connection ids"""
        self.point.figure.canvas.mpl_disconnect(self.cidpress)
        self.point.figure.canvas.mpl_disconnect(self.cidrelease)
        self.point.figure.canvas.mpl_disconnect(self.cidmotion)

# Diagramme U=f(t)
class Udiagram:
    def __init__(self, ax):
        """constructor"""
        global Tt, Tu
        self.ax = ax
        ax.set_xlim(0, Tmax)
        ax.set_ylim(-6, 6)
        plt.xlabel("t en ms")
        plt.ylabel("uc en V")
        ax.plot([0, Tmax], [0, 0], 'k--')
        self.uc, = ax.plot(Tt, Tu, label='uc', color='r')
        self.Uc, = ax.plot([0, Tmax], [Uc, Uc], label='<uc>', color='b')
        self.ly = ax.axvline(color='k')  # the black line
        self.Uc_value = Uc
        self.txt = ax.text(-2, 4, '')
        self.txt.set_text(
            '<uc>=%1.2f V, alpha=%1.2f, T=%1.0f ms' % (Uc, alpha, T))
        self.info = ax.text(500, -4, '', horizontalalignment='center',
                            bbox=dict(facecolor='red', alpha=0.5))
        self.info.set_text(
            'Déplacer à la souris le curseur\n pour animer le schéma')
        plt.legend()
        self.press = None

    def connect(self):
        """connect to all the events we need"""
        self.cidpress = self.uc.figure.canvas.mpl_connect(
            'button_press_event', self.on_press)
        self.cidrelease = self.uc.figure.canvas.mpl_connect(
            'button_release_event', self.on_release)
        self.cidmotion = self.uc.figure.canvas.mpl_connect(
            'motion_notify_event', self.on_motion)

    def on_press(self, event):
        """on button press we will see if the mouse is over us and store some data"""
        #print("on press")
        if event.inaxes != self.uc.axes:
            return

        #contains, attrd = self.uc.contains(event)
        # if not contains: return
        self.press = event.xdata, event.ydata
        self.info.set_text('')

    def on_motion(self, event):
        """"on motion we will move the point if the mouse is over us"""
        global nc, Tmax, N, tc
        if self.press is None:
            return
        if event.inaxes != self.uc.axes:
            return
        #xpress, ypress = self.press
        x = event.xdata
        y = event.ydata
        #print('{},{}'.format(event.xdata, event.ydata))
        tc = x
        nc = int((x/Tmax)*N)
        self.uc.figure.canvas.draw()
        update_dr1()

    def on_release(self, event):
        """on release we reset the press data"""
        self.press = None
        self.info.set_text(
            'Déplacer à la souris le curseur\n pour animer le schéma')
        self.info.figure.canvas.draw()

    def disconnect(self):
        """disconnect all the stored connection ids"""
        self.point.figure.canvas.mpl_disconnect(self.cidpress)
        self.point.figure.canvas.mpl_disconnect(self.cidrelease)
        self.point.figure.canvas.mpl_disconnect(self.cidmotion)

    def update(self):
        """update diagram"""
        global Uc, alpha, T
        global Tt, Tu
        self.Uc.set_ydata([Uc, Uc])
        self.txt.set_text(
            '<uc>=%1.2f V, alpha=%1.2f, T=%1.0f ms' % (Uc, alpha, T))
        self.uc.set_xdata(Tt)
        self.uc.set_ydata(Tu)


class Idiagram:
    def __init__(self, ax):
        """constructor"""
        self.ax = ax
        ax.set_xlim(0, Tmax)
        ax.set_ylim(-6, 6)
        plt.xlabel("t en ms")
        plt.ylabel("ic en A")
        ax.plot([0, Tmax], [0, 0], 'k--')
        self.ic, = ax.plot(Tt, Ti, label='ic', color='r')
        self.Ic, = ax.plot([0, Tmax], [Ic, Ic], label='<ic>', color='b')
        self.ly = ax.axvline(color='k')  # the black line
        self.txt = ax.text(-2, 4, '')
        self.txt.set_text('<ic>=%1.2f A' % (Uc))
        self.info = ax.text(0, -25, '', horizontalalignment='left',
                            bbox=dict(facecolor='red', alpha=0.5))
        self.info.set_text(
            'Déplacer les curseurs pour modifier les paramètres')
        plt.legend()
        self.press = None

        self.version = ax.text(1000, -25, '', horizontalalignment='right')
        self.version.set_text(
            'Lycée Livet (Nantes), Arnaud Dugas, arnaud.dugas@ac-nantes.fr, version {}'.format(version))
        self.version.set_fontsize(8)
        self.version.set_color('b')

    def connect(self):
        """connect to all the events we need"""
        self.cidpress = self.ic.figure.canvas.mpl_connect(
            'button_press_event', self.on_press)
        self.cidrelease = self.ic.figure.canvas.mpl_connect(
            'button_release_event', self.on_release)
        self.cidmotion = self.ic.figure.canvas.mpl_connect(
            'motion_notify_event', self.on_motion)

    def on_press(self, event):
        """on button press we will see if the mouse is over us and store some data"""
        #print("on press")
        if event.inaxes != self.ic.axes:
            return

        #contains, attrd = self.ic.contains(event)
        # if not contains: return
        self.press = event.xdata, event.ydata

    def on_motion(self, event):
        """on motion we will move the point if the mouse is over us"""
        global Tmax, N, nc, tc
        if self.press is None:
            return
        if event.inaxes != self.ic.axes:
            return
        #xpress, ypress = self.press
        x = event.xdata
        y = event.ydata
        #print('{},{}'.format(event.xdata, event.ydata))
        # self.uc.set_xdata(x)
        # self.uc.set_ydata(y)
        # self.ly.set_xdata(x)
        tc = x
        nc = int((x/Tmax)*N)
        self.ic.figure.canvas.draw()
        update_dr1()

    def on_release(self, event):
        """on release we reset the press data"""
        self.press = None
        # self.point.figure.canvas.draw()

    def disconnect(self):
        """disconnect all the stored connection ids"""
        self.point.figure.canvas.mpl_disconnect(self.cidpress)
        self.point.figure.canvas.mpl_disconnect(self.cidrelease)
        self.point.figure.canvas.mpl_disconnect(self.cidmotion)

    def update(self):
        """update diagram"""
        global Uc, alpha, T
        global Tt, Ti

        self.Ic.set_ydata([Ic, Ic])

        self.txt.set_text('<ic>=%1.2f A, (IM-Im)/2=%1.2f A' %
                          (Ic, (np.max(Ti)-np.min(Ti))/2))

        self.ic.set_xdata(Tt)
        self.ic.set_ydata(Ti)

# Class Schema


class SchemaElec:
    def __init__(self, ax):
        """constructor"""
        self.ax = ax
        self.info = ax.text(-30, 30, '', horizontalalignment='left')
        self.info.set_text(titre)
        self.info.set_fontsize(16)
        self.info.set_color('b')

        # éléments de dessin
        D1 = self.setDiode([0, 0])
        D2 = self.setDiode([60, 0])
        D3 = self.setDiode([0, -16])
        D4 = self.setDiode([60, -16])
        T1 = self.setTransistor([20, 0])
        T2 = self.setTransistor([20+60, 0])
        T3 = self.setTransistor([20, -16])
        T4 = self.setTransistor([20+60, -16])
        J11 = [[[0, 20], [16, 16]]]
        J12 = [[[20, 60], [16, 16]]]
        J13 = [[[60, 80], [16, 16]]]
        J21 = [[[0, 20], [0, 0]]]
        J23 = [[[60, 80], [0, 0]]]
        J31 = [[[0, 20], [-16, -16]]]
        J32 = [[[20, 60], [-16, -16]]]
        J33 = [[[60, 80], [-16, -16]]]
        J1x = J11+J12+J13
        J2x = J21+J23
        J3x = J31+J32+J33
        Jxx = J1x+J2x+J3x

        # motifs
        MCC = self.setMCC([40, 0])
        Gene = self.setGenerateur([-30, 0])
        self.Iplus = [[[25, 28, 25], [2, 0, -2]]]
        self.Imoins = [[[+55, 52, +55], [2, 0, -2]]]
        self.dessin = D1+D2+D3+D4+T1+T2+T3+T4+MCC+Jxx+Gene

        # Quadran 1 umoy>0,i>0
        self.T1T4 = T1+T4+Gene+MCC+J11+J23+J3x
        self.T1D2 = T1+D2+MCC+J12
        self.T4D3 = T4+D3+MCC+J2x+J3x

        # Quadran 2 umoy>0,i<0
        self.D1D4 = D1+D4+Gene+MCC+J21+J31+J32
        self.T2D1 = T2+J2x+D1+J1x + MCC
        self.T3D4 = T3+J32+D4 + MCC

        # Quadran 3 umoy<0,i<0
        self.T2T3 = T2+T3+Gene+MCC+J31+J23+J1x
        #self.T2D1 = T2+J2x+D1+J1x
        #self.T3D4 = T3+J32+D4

        # Quadran 4 umoy<0,i>0
        self.D2D3 = D2+D3+Gene+MCC+J21+J11+J12
        #self.T1D2 = T1+D2+MCC+J12
        #self.T4D3 = T4+D3+MCC+J2x+J3x

        # mise en page
        ax.set_xlim(-60, 90)
        ax.set_ylim(-20, 20)
        ax.axis('off')
        ax.axis('equal')
        self.signature = None  # pour l'optimisation de l'affichage

        # affichge du dessin
        self.plot(self.dessin)

        # affichage tensions et intensités
        plt.text(-45, 0, "U", fontsize=12, color='k',
                 horizontalalignment='center', verticalalignment='center')
        plt.arrow(-40, -15, 0, 30, head_width=2, head_length=2, fc='k', ec='k')
        self.uc = plt.text(40, 13, "uc", fontsize=12, color='w',
                           horizontalalignment='center', verticalalignment='center')
        self.uc_moins = plt.arrow(
            35, 9, 10, 0, head_width=2, head_length=2, fc='w', ec='w')
        self.uc_plus = plt.arrow(
            45, 11, -10, 0, head_width=2, head_length=2, fc='w', ec='w')
        # affichage légendes
        self.d1 = plt.text(-10, 10, "D1", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.d2 = plt.text(50, 10, "D2", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.d3 = plt.text(-10, -10, "D3", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.d4 = plt.text(50, -10, "D4", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.t1 = plt.text(10, 10, "T1", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.t2 = plt.text(70, 10, "T2", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.t3 = plt.text(10, -10, "T3", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.t4 = plt.text(70, -10, "T4", fontsize=12, color='k',
                           horizontalalignment='center', verticalalignment='center')
        self.mcc = plt.text(40, 0, "MCC", fontsize=12, color='k',
                            horizontalalignment='center', verticalalignment='center')

    def resetText(self):
        """initialisation de la couleur des textes"""
        self.d1.set(color='k', fontweight='normal')
        self.d2.set(color='k', fontweight='normal')
        self.d3.set(color='k', fontweight='normal')
        self.d4.set(color='k', fontweight='normal')
        self.t1.set(color='k', fontweight='normal')
        self.t2.set(color='k', fontweight='normal')
        self.t3.set(color='k', fontweight='normal')
        self.t4.set(color='k', fontweight='normal')
        self.uc.set(color='w', fontweight='normal')
        self.uc_moins.set(fc='w', ec='w')
        self.uc_plus.set(fc='w', ec='w')

    def resetDessin(self):
        """initialisation du dessin"""
        self.signature = None
        self.plot(self.Iplus, 'w-')
        self.plot(self.Imoins, 'w-')
        self.plot(self.dessin)

    def update(self):
        """mise à jour du diagramme"""
        global nc, Ti, Tu
        u = Tu[nc]
        i = Ti[nc]
        signature = (u == 0)+(u > 0)*10+(i > 0)*100
        if signature == self.signature:  # on ne fait rien si la signature u,i n'a pas changée
            return
        self.signature = signature
        self.plot(self.dessin)  # on efface
        self.resetText()  # on efface
        if u == 0:
            self.t1.set(color='g')
            self.t2.set(color='g')
            self.t3.set(color='r')
            self.t4.set(color='r')
            if i > 0:
                self.plot(self.Iplus, 'b-')
                self.plot(self.Imoins, 'w-')
                self.plot(self.T1D2, 'b-')
                self.t1.set(fontweight='bold')
                self.d2.set(color='b', fontweight='bold')
            else:
                self.plot(self.Iplus, 'w-')
                self.plot(self.Imoins, 'b-')
                self.plot(self.T2D1, 'b-')
                self.t2.set(fontweight='bold')
                self.d1.set(color='b', fontweight='bold')
        else:
            if u > 0:
                self.t1.set(color='g')
                self.t2.set(color='r')
                self.t3.set(color='r')
                self.t4.set(color='g')
                self.uc.set(color='k', fontweight='normal')
                self.uc_plus.set(fc='k', ec='k')
                if i > 0:
                    self.plot(self.Iplus, 'b-')
                    self.plot(self.Imoins, 'w-')
                    self.plot(self.T1T4, 'b-')
                    self.t1.set(fontweight='bold')
                    self.t4.set(fontweight='bold')
                else:
                    self.plot(self.Iplus, 'w-')
                    self.plot(self.Imoins, 'b-')
                    self.plot(self.D1D4, 'b-')
                    self.d1.set(color='b', fontweight='bold')
                    self.d4.set(color='b', fontweight='bold')
            if u < 0:
                self.t1.set(color='r')
                self.t2.set(color='g')
                self.t3.set(color='g')
                self.t4.set(color='r')
                self.uc.set(color='k', fontweight='normal')
                self.uc_moins.set(fc='k', ec='k')
                if i > 0:
                    self.plot(self.Iplus, 'b-')
                    self.plot(self.Imoins, 'w-')
                    self.plot(self.D2D3, 'b-')
                    self.d2.set(color='b', fontweight='bold')
                    self.d3.set(color='b', fontweight='bold')
                else:
                    self.plot(self.Iplus, 'w-')
                    self.plot(self.Imoins, 'b-')
                    self.plot(self.T2T3, 'b-')
                    self.t2.set(fontweight='bold')
                    self.t3.set(fontweight='bold')

    def plot(self, data, couleur='k'):
        """dessin de n motifs du type [LXn,LXn]"""
        # data formaté [[LX1,LX2],[LY1,LY2],...]
        for elt in data:
            self.ax.plot(elt[0], elt[1], couleur)

    def setDiode(self, position):
        """création d'un motif de type diode"""
        # hauteur 16
        x = np.array([0, 0, 5, -5, 0, 5, -5, 0, 0])+position[0]
        y = np.array([0, 12, 4, 4, 12, 12, 12, 12, 16])+position[1]
        return [[x, y]]

    def setTransistor(self, position):
        """création d'un motif de type transistor"""
        # hauteur 16
        x = np.array([0, 0, 5, 0, 5, -5, -3, -5, -3, -
                     1, -3, -1, 0, -5, 0, 0])+position[0]
        y = np.array([0, 12, 12, 4, 4, 4, 4, 2, 4, 4,
                     2, 4, 4, 12, 12, 16])+position[1]
        return [[x, y]]

    def setMCC(self, position):
        """création d'un motif de type MCC"""
        # largeur 40
        theta = np.linspace(0, 2*np.pi, 100)
        r = 7.5
        x1 = r*np.cos(theta)+position[0]
        y1 = r*np.sin(theta)+position[1]
        x2 = np.array([-20, -7.5])+position[0]
        y2 = np.array([0, 0])+position[1]
        x3 = np.array([20, 7.5])+position[0]
        y3 = np.array([0, 0])+position[1]
        return([[x1, y1], [x2, y2], [x3, y3]])

    def setGenerateur(self, position):
        """création d'un motif de type générateur"""
        # largeur 30 hauteur 32
        theta = np.linspace(0, 2*np.pi, 100)
        r = 7.5
        x1 = r*np.cos(theta)+position[0]
        y1 = r*np.sin(theta)+position[1]
        x2 = np.array([30, 0, 0, 30])+position[0]
        y2 = np.array([-16, -16, 16, 16])+position[1]

        return([[x1, y1], [x2, y2]])

# Class Schema Méca


class SchemaMeca:
    def __init__(self, ax):
        """constructor"""
        self.ax = ax

        # éléments de dessin
        Poulie = self.setPoulie([0, 0])
        Poids_droite = self.setPoids([15, 0])
        Poids_gauche = self.setPoids([-15, 0])
        Bati = self.setBati([0, 0])

        # motifs
        self.poids_gauche = Poulie + Poids_gauche + Bati
        self.poids_droite = Poulie + Poids_droite + Bati
        self.sens_trigo = self.setFlecheVitesse([0, 0], "trigo")
        self.sens_horaire = self.setFlecheVitesse([0, 0], "horaire")

        self.signature = None  # pour l'optimisation de l'affichage

        # affichge du dessin
        self.plot(self.poids_droite)
        self.plot(self.sens_trigo, couleur='b')

        # affichage légendes
        self.ax.text(25, 25, "w > 0", fontsize=12, color='b',
                     horizontalalignment='center', verticalalignment='center')
        self.ax.text(0, -40, "C > 0", fontsize=12, color='k',
                     horizontalalignment='center', verticalalignment='center')

    def update(self):
        """mise à jour du diagramme"""
        global Uc, Ic
        u = Uc
        i = Ic
        signature = (u == 0)+(u > 0)*10+(i > 0)*100
        if signature == self.signature:  # on ne fait rien si la signature u,i n'a pas changée
            return
        self.signature = signature
        self.ax.clear()
        if u == 0:
            if i > 0:
                self.ax.text(0, -40, "C > 0", fontsize=12, color='k',
                             horizontalalignment='center', verticalalignment='center')
                self.plot(self.poids_droite)
            else:
                self.ax.text(0, -40, "C < 0", fontsize=12, color='k',
                             horizontalalignment='center', verticalalignment='center')
                self.plot(self.poids_gauche)
        else:
            if u > 0:
                self.ax.text(25, 25, "w > 0", fontsize=12, color='b',
                             horizontalalignment='center', verticalalignment='center')
                self.plot(self.sens_trigo, couleur='b')
                if i > 0:
                    self.ax.text(0, -40, "C > 0", fontsize=12, color='k',
                                 horizontalalignment='center', verticalalignment='center')
                    self.plot(self.poids_droite)
                else:
                    self.ax.text(0, -40, "C < 0", fontsize=12, color='k',
                                 horizontalalignment='center', verticalalignment='center')
                    self.plot(self.poids_gauche)
            if u < 0:
                self.plot(self.sens_horaire, couleur='b')
                self.ax.text(25, 25, "w < 0", fontsize=12, color='b',
                             horizontalalignment='center', verticalalignment='center')
                if i > 0:
                    self.ax.text(0, -40, "C > 0", fontsize=12, color='k',
                                 horizontalalignment='center', verticalalignment='center')
                    self.plot(self.poids_droite)
                else:
                    self.ax.text(0, -40, "C < 0", fontsize=12, color='k',
                                 horizontalalignment='center', verticalalignment='center')
                    self.plot(self.poids_gauche)

    def plot(self, data, couleur='k'):
        """dessin de n motifs du type [LXn,LXn]"""
        # data formaté [[LX1,LX2],[LY1,LY2],...]

        # mise en page
        self.ax.set_xlim(-50, 50)
        self.ax.set_ylim(-40, 20)
        self.ax.axis('off')
        self.ax.axis('equal')

        for elt in data:
            self.ax.plot(elt[0], elt[1], couleur)

    def setPoulie(self, position):
        """création d'un motif de type Poulie"""
        # largeur 40
        theta = np.linspace(0, 2*np.pi, 100)
        r = 15
        x1 = r*np.cos(theta)+position[0]
        y1 = r*np.sin(theta)+position[1]
        x2 = np.array([2.5/np.sqrt(2), r/np.sqrt(2)])+position[0]
        y2 = np.array([2.5/np.sqrt(2), r/np.sqrt(2)])+position[1]
        return([[x1, y1], [x2, y2]])

    def setBati(self, position):
        """création d'un motif de type Poulie"""
        # largeur 40
        theta = np.linspace(0, 2*np.pi, 100)
        r = 2.5
        x1 = r*np.cos(theta)+position[0]
        y1 = r*np.sin(theta)+position[1]
        x2 = np.array([0, 0, -5, 5])+position[0]
        y2 = np.array([-2.5, -5, -5, -5])+position[1]
        x = np.array([-5, -7]) + position[0]
        y = np.array([-5, -7]) + position[1]
        l3 = [[x + 2*i, y] for i in range(6)]

        return([[x1, y1], [x2, y2]]+l3)

    def setPoids(self, position):
        """création d'un motif de type Poids"""
        x1 = np.array([0, 0])+position[0]
        y1 = np.array([0, -20])+position[1]
        x2 = np.array([-5, 5, 7, -7, -5])+position[0]
        y2 = np.array([-20, -20, -30, -30, -20])+position[1]

        return([[x1, y1], [x2, y2]])

    def setFlecheVitesse(self, position, sens="trigo"):
        """création d'un motif de type fleche vitesse"""
        # largeur 40
        if sens == "trigo":
            signe = 1
        else:
            signe = -1

        theta = np.linspace(0, np.pi/2, 100)
        r = 20
        x1 = signe*r*np.cos(theta)+position[0]
        y1 = r*np.sin(theta)+position[1]

        x2 = [signe*4, 0, signe*3]
        y2 = [r+2, r, r-3]

        return([[x1, y1], [x2, y2]])

# Mises à jour
def updateF(val):
    """actualisation si changement de F"""
    global T
    T = 1000/val  # en ms
    updateAll()
    reset_dr1()


def updateL(val):
    """actualisation si changement de L"""
    global L
    L = val  # en mH
    updateAll()
    reset_dr1()


def updateR(val):
    """actualisation si changement de R"""
    global R
    R = val  # en ohm
    updateAll()
    reset_dr1()

def updateMode(label):
    global mode, titre

    mode = label #unipolaire ou bipolaire
    if mode == "unipolaire":
        titre = "Hacheur en commande unipolaire"
    else:
        titre = "Hacheur en commande bipolaire"
    dr1.info.set_text(titre)
    updateAll()
    update_dr1()
    update_dr5()

def updateData():
    """actualisations des données globales Tu, Ti et Tt"""
    # On calcul sur une durée double de celle affiché (2Tt) et on calcul donc 2*N pts
    # afin de s'affranchir du régime transitoire

    global Uc, Ic, Umax, alpha
    global Tt, Tu, Ti

    if mode == "unipolaire":
        alpha = abs(Uc/Umax)
        if Uc > 0:
            u1 = Umax
        else:
            u1 = Umin
        u2 = 0
    else:
        alpha = (Uc/Umax+1)/2
        u1 = Umax
        u2 = Umin

    n = 2*N
    tmax = 2*Tmax
    # Calcul du tableau des tensions et des temps
    k = 0  # Numéro de Période
    for j in range(0, n):
        Tt_c[j] = j*tmax/(n-1)
        if j*tmax/(n-1) < (alpha*T+k*T):
            Tu_c[j] = u1
        else:
            Tu_c[j] = u2
        if j*tmax/(n-1) > (k+1)*T:
            k += 1
    # Calcul du tableau des i
    # L'équation différentielle sous forme de fonction :

    def f(i, u):
        # di/dt = f(i,u)
        global R, L, Uc

        return 1000/L*(u-Uc-R*i)

    # On résout notre équation différentielle et on récupère la liste des résultats
    #Ti=odeint(f, 0, Tt)
    Ti_c[0] = 0
    pas = (Tt_c[1]-Tt_c[0])/1000 # /1000 conversion en secondes
    for j in range(1, n):
        Ti_c[j] = Ti_c[j-1]+pas*f(Ti_c[j-1], Tu_c[j-1])
    # Extaction de la zone utile
    Tt = Tt_c[N:2*N] - Tmax
    Ti = Ti_c[N:2*N]
    Tu = Tu_c[N:2*N]
    # Ajustement de la moyenne
    Ti = Ti + Ic - np.mean(Ti)

def updateAll():
    """mise à jour de dr3 u=f(t) et dr4 i=f(t)"""
    updateData()
    dr3.update()  # Actualisation graph u=f(t)
    dr4.update()  # Actualisation graph i=f(t)



def update_dr1():
    """mise à jour de dr1 (schéma élec)"""
    global tc
    dr3.ly.set_xdata(tc)
    dr4.ly.set_xdata(tc)
    dr1.update()


def reset_dr1():
    """réinitialisation du dr1 (schéma élec)"""
    if dr1.signature != None:
        dr1.resetDessin()
        dr1.resetText()


def update_dr5():
    """mise à jour de dr5 (schéma méca)"""
    dr5.update()

# ***************************************************************************
# Main
# ***************************************************************************
# dr1 : Schéma électrique
# dr2 : Diagram UI
# dr3 : U = f(t)
# dr4 : I = f(t)
# dr5 : Schéma Méca


if __name__ == "__main__":
    updateData()  # initialisation
    fig = plt.figure(figsize=(12, 8))
    #titre = plt.title('{} (v{})'.format(titre,version))
    # titre.set_fontsize(16)
    # titre.set_color('b')

    # dr2 diagram u,i
    ax2 = fig.add_subplot(443)
    dr2 = UIdiagram(ax2)
    dr2.connect()
    # dr5 Schéma Méca
    ax5 = fig.add_subplot(444)
    dr5 = SchemaMeca(ax5)

    # ajustement marges
    plt.subplots_adjust(left=0.125,
                        bottom=0.1,
                        right=0.9,
                        top=0.9,
                        wspace=0.2,
                        hspace=0.35)
    # dr1 Schéma Elec
    ax1 = fig.add_subplot(421)
    dr1 = SchemaElec(ax1)

    # dr3 diagram u=f(t)
    ax3 = fig.add_subplot(412)
    dr3 = Udiagram(ax3)
    dr3.connect()
    # dr4 diagram i=f(t)
    ax4 = fig.add_subplot(413)
    dr4 = Idiagram(ax4)
    dr4.connect()
    # https://matplotlib.org/stable/tutorials/intermediate/arranging_axes.html
    # https://www.delftstack.com/fr/howto/matplotlib/how-to-make-different-subplot-sizes-in-matplotlib/
    axcolor = 'lightgoldenrodyellow'

    axfreq = fig.add_axes([0.30, 0.2, 0.6, 0.03], facecolor=axcolor)
    sfreq = Slider(axfreq, 'Freq (kHz)', 1, 50, valinit=Tmax/T)
    sfreq.on_changed(updateF)

    axL = fig.add_axes([0.30, 0.15, 0.6, 0.03], facecolor=axcolor)
    sL = Slider(axL, 'L (mH)', 1, 50, valinit=L)
    sL.on_changed(updateL)

    axR = fig.add_axes([0.30, 0.10, 0.6, 0.03], facecolor=axcolor)
    sR = Slider(axR, 'R (ohm)', 0.1, 3, valinit=R)
    sR.on_changed(updateR)

    rax = fig.add_axes([0.11, 0.1, 0.10, 0.10], facecolor=axcolor)
    radio = RadioButtons(rax, ('unipolaire', 'bipolaire'))
    radio.on_clicked(updateMode)


    update_dr1()
    plt.show()
